﻿using System.Linq;
using System.Windows;
using System.Xml.Linq;
 
using Infragistics.Persistence;
using Infragistics.Controls.Grids;
using System;
using System.IO;
using System.Windows.Data;
using System.Reflection;
using Infragistics.Samples.Shared.Models.Dkb;
using System.Windows.Controls;
using Infragistics.Samples.Shared.Extensions.Dkb;
using DkbGrid.Resources;

namespace IGGrid.Samples.Organization
{
    public partial class SummaryRowFormatting : UserControl
    {
        public SummaryRowFormatting()
        {
            GridStrings Test = new GridStrings();
            InitializeComponent();
            //this.Loaded += OnSampleLoademd;
        }

        private void UserControl_Loaded(object sender, RoutedEventArgs e)
        {
            OnSampleLoaded(null, null);
        }
        void OnSampleLoaded(object sender, RoutedEventArgs e)
        {
            DownloadDataSource(@"Data\Properties.xml");
        }

        private void DownloadDataSource()
        { 
        }

        void DownloadDataSource(string p_Path) //(object sender, GetXmlDataCompletedEventArgs e)
        {
            //if (e.Error != null) return;


            string codeBase = Assembly.GetExecutingAssembly().CodeBase;
            UriBuilder uri = new UriBuilder(codeBase);
            string path = Uri.UnescapeDataString(uri.Path);



            XDocument doc = XDocument.Load(Path.Combine(Path.GetDirectoryName(path) , p_Path));
            var dataSource = (from d in doc.Descendants("Property")
                              select new RealEstate
                              {
                                  MLS = d.Attribute("MLS").GetString(),
                                  ListingDate = d.Attribute("ListingDate").GetDateTime(),
                                  NumberOfBedRooms = d.Attribute("NumberOfBedRooms").GetInt(),
                                  NumberOfBathRooms = d.Attribute("NumberOfBathRooms").GetInt(),
                                  SqFeet = d.Attribute("SqFeet").GetDecimal(),
                                  Price = d.Attribute("Price").GetDouble(),
                                  City = d.Attribute("City").GetString(),
                                  State = d.Attribute("State").GetString(),
                                  PostalCode = d.Attribute("PostalCode").GetString()
                              });

            this.dataGrid.ItemsSource = dataSource.ToList();
        }

        /// <summary>
        /// Access to resource dictionary entry for the persistence group used for persistence
        /// </summary>
        private PersistenceGroup XamGridColumnPersistenceGroup
        { get { return (PersistenceGroup)Resources["XamGridColumnPersistenceGroup"]; } }



        /// <summary>
        /// Access to resource dictionary entry for special persistence of column layouts 
        /// (columns visible, column positioning, etc).
        /// </summary>
        private PersistenceSettings ColumnLayoutPersistenceSettings
        { get { return (PersistenceSettings)Resources["ColumnLayoutPersistenceSettings"]; } }

        /// <summary>
        /// Access to resource dictionary entry for special persistence .
        /// </summary>
        private PersistenceSettings AllSavedPersistenceSettings
        { get { return (PersistenceSettings)Resources["AllSavedPersistenceSettings"]; } }


        /// <summary>
        /// Clear from column layouts and columns.
        /// </summary>
        private void ClearPersistenceSettings()
        {
            // No attached properties on grid
            dataGrid.SetValue(PersistenceManager.PersistenceGroupProperty, DependencyProperty.UnsetValue);
            dataGrid.SetValue(PersistenceManager.SettingsProperty, DependencyProperty.UnsetValue);

            // No attached properties on layouts
            foreach (ColumnLayout LoopLayout in dataGrid.ColumnLayouts)
            {

                LoopLayout.SetValue(PersistenceManager.PersistenceGroupProperty, DependencyProperty.UnsetValue);
                LoopLayout.SetValue(PersistenceManager.SettingsProperty, DependencyProperty.UnsetValue);
            }

            // No attached properties on columns
            foreach (ColumnBase LoopColumn in dataGrid.Columns)
            {
                LoopColumn.SetValue(PersistenceManager.PersistenceGroupProperty, DependencyProperty.UnsetValue);
                LoopColumn.SetValue(PersistenceManager.SettingsProperty, DependencyProperty.UnsetValue);
            }
        }

        /// <summary>
        /// Operates on column layouts.
        /// </summary>
        private void InitColumnLayoutPersistenceSettings(bool p_Loading, out PersistenceGroup p_PersistenceGroup)
        {
            // Send this group back.
            p_PersistenceGroup = XamGridColumnPersistenceGroup;

            // Clear before anything else
            ClearPersistenceSettings();

            // Default local settings reference
            PersistenceSettings SettingsObj = ColumnLayoutPersistenceSettings;

            // If loading, deserialize everything present
            if (p_Loading) { SettingsObj = AllSavedPersistenceSettings; }

            // Single column layout expected                
            if (dataGrid.ColumnLayouts.Count != 1)
            { throw new Exception("This method only supports a single layout."); }

            // Assign the attached properties now.
            PersistenceManager.SetPersistenceGroup(dataGrid.ColumnLayouts.First(), XamGridColumnPersistenceGroup);
            PersistenceManager.SetSettings(dataGrid.ColumnLayouts.First(), SettingsObj);


        }

        private XDocument m_SavedResultsDocument = new XDocument();

        private void m_BtnSave_Click(object sender, RoutedEventArgs e)
        {
            // Locals
            MemoryStream StreamObj;
            PersistenceGroup SaveGroup = null;
            StreamReader SettingsReaderObj;
            XDocument XmlResultsDocument;

            // Init for saving
            InitColumnLayoutPersistenceSettings(p_Loading: false, p_PersistenceGroup: out SaveGroup);

            // Save with specified settings
            StreamObj = new MemoryStream();
            StreamObj = PersistenceManager.Save(SaveGroup);
            SettingsReaderObj = new StreamReader(StreamObj);

            // Reset stream position and 
            // load settings into XML doc
            SettingsReaderObj.BaseStream.Position = 0;
            XmlResultsDocument = XDocument.Load(SettingsReaderObj);
            m_SavedResultsDocument = XmlResultsDocument;

        }

        private void m_BtnLoad_Click(object sender, RoutedEventArgs e)
        {

            // Local variables
            string MessageBoxTitle;
            string MessageBoxContent;
            Stream StreamObj = null;
            PersistenceGroup LoadGroup = null;
            bool ColumnResetDetected = false; 


            try // ... and return status of visibility
            {
                // Load the stream
                StreamObj = new MemoryStream();
                m_SavedResultsDocument.Save(StreamObj);
                StreamObj.Position = 0;

                // Init for column layout loading.  Provides us with persistence group to use.
                InitColumnLayoutPersistenceSettings(p_Loading: true, p_PersistenceGroup: out LoadGroup);

                // Load the grid column layout
                // (May throw InvalidPersistenceGroupException or others)
                PersistenceManager.Load(LoadGroup, StreamObj);

            }
            catch(Exception Ex)
            {

                System.Diagnostics.Debug.WriteLine("Message " + Ex.ToString());
                System.Diagnostics.Debugger.Break(); //.WriteLine("Message " + Ex.ToString());
                throw;
            }
        }

    }
}
